#!/usr/bin/env python3
import os
import problem as p
import unittest
import json

TEST_DIRECTORY = os.path.dirname(__file__)

##################################################
##  Problem 01 Tests
##################################################
class TestProblem01(unittest.TestCase):

    def test_1_single_elt(self):
        tests = [[0], [1], [3], [3.1415926], [2.718281828], [100000000]]
        exps = [0.0, 1.0, 3.0, 3.1415926, 2.718281828, 100000000.0]
        for test, exp in zip(tests, exps):
            res = p.average(test)
            self.assertEqual(res, exp, "Неправильное среднее. Ожидается %s, ваша функция посчитала %s" %(exp, res))
    
    def test_2_small_ints(self):
        tests = [[449, 318, 610, 325, 782], [600, 555, 171, 498, 155, 304], [578, 46, 102, 998, 944, 476, 164, 179, 219, 411], [391, 350, 382, 739, 552, 84, 537, 839], [177, 821, 311, 843, 831]]
        exps = [496.8, 380.5, 411.7, 484.25, 596.6]
        for test, exp in zip(tests, exps):
            res = p.average(test)
            self.assertAlmostEqual(res, exp, "Неправильное среднее. Ожидается %s, ваша функция посчитала %s" %(exp, res))

    def test_3_small_floats(self):
        tests = [[0.47203083150348724, 0.17434572831129536, 1.6575439713590696, 0.5872336701806269, 0.6466430429022682, 1.4835408548727598], [1.466139000242515, 1.4773003591227474, 2.834279903090738, 0.826612205088214, 1.637643732456928, 1.9826429207049552, 2.624264180424307, 2.4191835259942365, 0.6760582993714013], [0.854717427511195, 1.5753843726483705, 2.727641586555019, 0.7563096314948793, 1.5645752742777705, 0.6026157427384358], [2.640935945832427, 0.22406999326525034, 0.3348669174323774, 0.6734022945679214, 1.135478955876866, 2.6632629874123714, 2.040652221251602, 1.8237187655303022, 1.6030656389871685], [0.26429979928039293, 2.800972882994899, 2.1448317222521442, 0.8890438257133574, 0.7866992537734674, 2.10507790241449, 2.416035265905575, 0.9297122489933412, 1.7381591521886341]]
        exps = [0.8368896831882511, 1.771569347388449, 1.346874005870945, 1.4599393022395872, 1.5638702281684782]
        for test, exp in zip(tests, exps):
            res = p.average(test)
            self.assertAlmostEqual(round(res, 5) , round(exp, 5), "Неправильное среднее. Ожидается %s, ваша функция посчитала %s" %(round(exp, 5), round(res, 5)))

    def test_4_large(self):
        with open('test_inputs/test4.json') as f:
            tests = json.load(f)
        exps = [1.4469292350012337, 1.237853611519972, 1.3041404985352032]
        for test, exp in zip(tests, exps):
            res = p.average(test)
            self.assertAlmostEqual(round(res, 5) , round(exp, 5), "Неправильное среднее. Ожидается %s, ваша функция посчитала %s" %(round(exp, 5), round(res, 5)))

##################################################
##  Problem 02 Tests
##################################################
class TestProblem02(unittest.TestCase):

    def test_is_permutation_1(self):
        A, B = "cat", "act"
        exp = True
        res = p.is_permutation(A,B)
        self.assertEqual(res, exp, 'Ожидалось %s, ваша функция вернула %s' %(exp, res))

    def test_is_permutation_2(self):
        A, B = "", ""
        exp = True
        res = p.is_permutation(A,B)
        self.assertEqual(res, exp, 'Ожидалось %s, ваша функция вернула %s' %(exp, res))

    def test_is_permutation_3(self):
        A, B = "caterpillar", "pillarcatir"
        exp = False
        res = p.is_permutation(A,B)
        self.assertEqual(res, exp, 'Ожидалось %s, ваша функция вернула %s' %(exp, res))

    def test_is_permutation_4(self):
        A, B = "BeyondCurriculum", "dnoyBeCuicumrlur"
        exp = True
        res = p.is_permutation(A,B)
        self.assertEqual(res, exp, 'Ожидалось %s, ваша функция вернула %s' %(exp, res))

    def test_is_permutation_5(self):
        A, B = "finaltest", "final test"
        exp = False
        res = p.is_permutation(A,B)
        self.assertEqual(res, exp, 'Ожидалось %s, ваша функция вернула %s' %(exp, res))


##################################################
##  Problem 03 Tests
##################################################
class TestProblem03(unittest.TestCase):
    def test_mode_1(self):
        test = [3, 4, 2, 3, 4, 3]
        exp = 3
        res = p.mode(test)
        self.assertEqual(res, exp, 'Ожидалось %s, ваша функция вернула %s' %(exp, res))

    def test_mode_2(self):
        test = [2, 1, 4, 4, 3, 4]
        exp = 4
        res = p.mode(test)
        self.assertEqual(res, exp, 'Ожидалось %s, ваша функция вернула %s' %(exp, res))
    
    def test_mode_3(self):
        test = [3, 4, 4, 2, 3, 2, 2, 4, 4]
        exp = 4
        res = p.mode(test)
        self.assertEqual(res, exp, 'Ожидалось %s, ваша функция вернула %s' %(exp, res))
        test = [2, 3, 2, 2, 4, 4, 2, 4, 2]
        exp = 2
        res = p.mode(test)
        self.assertEqual(res, exp, 'Ожидалось %s, ваша функция вернула %s' %(exp, res))

    def test_mode_4(self):
        test = [9, 3, 5, 4, 10, 4, 10, 5, 4, 7, 2, 2, 5, 4, 7, 9, 3, 5, 5, 8, 7, 4, 3, 7, 9, 1, 10, 9, 6, 2, 7, 1, 5, 10, 3, 5, 8, 4, 7, 4, 7, 3, 2, 3, 3, 6, 3, 2, 10, 6, 10, 10, 9, 5, 7, 10, 4, 5, 3, 10, 3, 10, 9, 10, 5, 1, 2, 8, 9, 7, 4, 10, 4, 3, 2, 8, 3, 5, 6, 2, 5, 6, 7, 2, 2, 9, 3, 1, 4, 8, 5, 2, 7, 1, 1, 7, 10, 2, 9, 8]
        exp = (3, 5, 10)
        res = p.mode(test)
        self.assertIn(res, exp, 'Ожидалось что-то из %s, ваша функция вернула %s' %(exp, res))

if __name__ == '__main__':
    #this part is important for providing helpful info
    res = unittest.main(verbosity=3, exit=False)